#
# Example script for running on Fram
#

#!/bin/bash -l

#SBATCH --account=nn9481k
#SBATCH --job-name=NC0a08
#SBATCH -t 01:00:00
#SBATCH --qos=preproc
#SBATCH -N 1   # number of nodes
#SBATCH -n 24 # number of cores
#SBATCH  --mail-type=END
#SBATCH --mail-user=annette.samuelsen@nersc.no

#SBATCH -o log/mpijob.out
#SBATCH -e log/mpijob.err

#
#  Give the job a name
#
#         
module restore system
module load NCL/6.4.0-intel-2017a
module load FFTW/3.3.6-intel-2017a


export NMPI=24
export SLURM_SUBMIT_DIR=$(pwd)
# Enter directory from where the job was submitted
cd $SLURM_O_WORKDIR       ||  { echo "Could not go to dir $SLURM_O_WORKDIR  "; exit 1; }

# ------------------- Fetch Environment ------------------------------
# -------- these are needed in preprocess scripts---------------------
echo "SLURM_JOBID    = $SLURM_JOBID     "
echo "SLURM_JOBNAME  = $SLURM_JOBNAME   "
echo "SLURM_SUBMIT_DIR= $SLURM_SUBMIT_KDIR "
echo "SLURM_TASKNUM  = $SLURM_TASKNUM "
echo "SLURM_NUM_PPN  = $SLURM_NUM_PPN "
[ -z "$NOMP" ] && NOMP=0

# Enter directory from where the job was submitted
cd $SLURM_SUBMIT_DIR       ||  { echo "Could not go to dir $SLURM_O_WORKDIR  "; exit 1; }

# Initialize environment (sets Scratch dir ($S), Data dir $D ++ )
source ../REGION.src  || { echo "Could not source ../REGION.src "; exit 1; }
source ./EXPT.src  || { echo "Could not source EXPT.src"; exit 1; }
echo "NMPI =$NMPI (Number of MPI tasks needed for running job) "

./preprocess.sh
# Enter Scratch/run dir and Run model
cd $S  ||  { echo "Could not go to dir $S  "; exit 1; }
#srun -n $NMPI ./hycom_cice  > ../log/hycom.${SLURM_JOBID}.out 2>&1
mpirun -n $NMPI ./hycom

# Cleanup and move data files to data directory - must be in "expt_XXX" dir for this script
cd $P     ||  { echo "Could not go to dir $P  "; exit 1; }
./postprocess.sh 

exit $?

