#!/bin/bash
# Set up 2d tiling for region

# Input args
if [ $# -ne 3 ]  ; then
    echo "This script will set up the partition files needed when running HYCOM with"
    echo "MPI parallelization. The input is the number of partitions along the 1st"
    echo "and 2nd dimensions, and the topography version to apply the partitioning to"
    echo
    echo "Usage:" 
    echo "   $(basename $0) [1st tile dimension] [2nd tile dimension] [topo version]" 
    echo "Example:" 
    echo "   $(basename $0) -2 -2 01 " 
    echo "  tip  - two negative tile dimensions gives a uniform grid "
    exit 1
fi
iqr=$1
jqr=$2
T=$3



# Set basedir based on relative paths of script
# Can be troublesome, but should be less prone to errors
# than setting basedir directly
cd $(dirname $0)/../../ 
export BASEDIR=$(pwd)/  
cd -

source ${BASEDIR}/REGION.src || { echo "Could not source ${BASEDIR}/REGION.src" ; exit 1 ; }

# Check that pointer to HYCOM_ALL is set (from EXPT.src)
if [ -z ${HYCOM_ALL} ] ; then
   echo "Environment not set "
   exit
else
   if [ ! -d ${HYCOM_ALL} ] ; then
      echo "HYCOM_ALL not properly set up"
      echo "HYCOM_ALL not a directory at ${HYCOM_ALL}"
      exit
   fi
fi

# Pointers to programs
export PARTIT=${HYCOM_ALL}/topo/src/partit
export TOPO_PPM=${HYCOM_ALL}/topo/src/topo_ppm


# Create work dir, and copy files to it
mkdir -p $BASEDIR/topo/partit/SCRATCH 
cd $BASEDIR/topo/partit/SCRATCH || \
   { echo "Could not descend dir  $BASEDIR/topo/partit/SCRATCH" ; exit 1 ;}
touch regional.grid.a regional.grid.b regional.depth.a regional.depth.b fort.21 fort.31
rm    regional.grid.a regional.grid.b regional.depth.a regional.depth.b fort.21 fort.31
cp  $BASEDIR/topo/regional.grid.a . || { echo "Could not copy regional.grid.a " ; exit 1 ;}
cp  $BASEDIR/topo/regional.grid.b . || { echo "Could not copy regional.grid.b " ; exit 1 ;}
cp  $BASEDIR/topo/depth_${R}_${T}.a regional.depth.a || \
   { echo "Could not copy depth_${R}_${T}.a " ; exit 1 ;}
cp  $BASEDIR/topo/depth_${R}_${T}.b regional.depth.b || \
   { echo "Could not copy depth_${R}_${T}.b " ; exit 1 ;}
cp  $BASEDIR/topo/bin/xbathy.pal  . ||  { echo "Could not copy xbathy.pal " ; exit 1 ;}

# The tiler program needs these
export FOR051=regional.depth.b
export FOR051A=regional.depth.a
export FOR021=fort.21

# Calculate "load-balancing" patches
echo "$iqr $jqr 0.75" | $PARTIT
nm=$(head -n2 fort.21 | tail -n1 | tr -s  " " | cut -d " " -f 2)
#nm=$(printf "%4.4d" $nm)
nm=`echo 000$nm | tail -5c`

cp fort.21 depth_${R}_${T}.$nm

# Create graphics
$TOPO_PPM
mv fort.31 depth_${R}_${T}.$nm.ppm
rm fort.*

# Show it if ImageMagick (display) is present...
echo "Output partition file is  depth_${R}_${T}.$nm, with $nm tiles"
which display &> /dev/null && display  depth_${R}_${T}.$nm.ppm

# Copy partitions to parent directory - overwrite if present
mv depth_${R}_${T}.$nm.ppm depth_${R}_${T}.$nm $BASEDIR/topo/partit

echo
echo "partition file depth_${R}_${T}.$nm placed in $BASEDIR/topo/partit"
